@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -53,7 +53,7 @@ class DebugViewer(QWidget):
         """
         Constructor
         
-        @param debugServer reference to the debug server object
+        @param debugServer reference to the debug server object (DebugServer)
         @param docked flag indicating a dock window
         @param vm reference to the viewmanager object
         @param parent parent widget (QWidget)
@@ -108,7 +108,7 @@ class DebugViewer(QWidget):
         self.glvWidgetVLayout.setSpacing(3)
         self.glvWidget.setLayout(self.glvWidgetVLayout)
         
-        self.globalsViewer = VariablesViewer(self.glvWidget, True)
+        self.globalsViewer = VariablesViewer(self, True, self.glvWidget)
         self.glvWidgetVLayout.addWidget(self.globalsViewer)
         
         self.glvWidgetHLayout = QHBoxLayout()
@@ -138,8 +138,8 @@ class DebugViewer(QWidget):
         self.__tabWidget.setTabToolTip(index, self.globalsViewer.windowTitle())
         
         self.setGlobalsFilterButton.clicked.connect(
-            self.__setGlobalsFilter)
-        self.globalsFilterEdit.returnPressed.connect(self.__setGlobalsFilter)
+            self.setGlobalsFilter)
+        self.globalsFilterEdit.returnPressed.connect(self.setGlobalsFilter)
         
         # add the local variables viewer
         self.lvWidget = QWidget()
@@ -161,7 +161,7 @@ class DebugViewer(QWidget):
         self.sourceButton.setEnabled(False)
         self.lvWidgetVLayout.addLayout(self.lvWidgetHLayout1)
 
-        self.localsViewer = VariablesViewer(self.lvWidget, False)
+        self.localsViewer = VariablesViewer(self, False, self.lvWidget)
         self.lvWidgetVLayout.addWidget(self.localsViewer)
         
         self.lvWidgetHLayout2 = QHBoxLayout()
@@ -194,8 +194,8 @@ class DebugViewer(QWidget):
         self.sourceButton.clicked.connect(self.__showSource)
         self.stackComboBox.currentIndexChanged[int].connect(
             self.__frameSelected)
-        self.setLocalsFilterButton.clicked.connect(self.__setLocalsFilter)
-        self.localsFilterEdit.returnPressed.connect(self.__setLocalsFilter)
+        self.setLocalsFilterButton.clicked.connect(self.setLocalsFilter)
+        self.localsFilterEdit.returnPressed.connect(self.setLocalsFilter)
         
         from .CallStackViewer import CallStackViewer
         # add the call stack viewer
@@ -305,8 +305,8 @@ class DebugViewer(QWidget):
         """
         self.globalsViewer.handleResetUI()
         self.localsViewer.handleResetUI()
-        self.__setGlobalsFilter()
-        self.__setLocalsFilter()
+        self.setGlobalsFilter()
+        self.setLocalsFilter()
         self.sourceButton.setEnabled(False)
         self.currentStack = None
         self.stackComboBox.clear()
@@ -461,17 +461,17 @@ class DebugViewer(QWidget):
         if self.__autoViewSource:
             self.__showSource()
         
-    def __setGlobalsFilter(self):
+    def setGlobalsFilter(self):
         """
-        Private slot to set the global variable filter.
+        Public slot to set the global variable filter.
         """
         filter = self.globalsFilterEdit.text()
         self.debugServer.remoteClientSetFilter(1, filter)
         self.debugServer.remoteClientVariables(2, self.globalsFilter)
         
-    def __setLocalsFilter(self):
+    def setLocalsFilter(self):
         """
-        Private slot to set the local variable filter.
+        Public slot to set the local variable filter.
         """
         filter = self.localsFilterEdit.text()
         self.debugServer.remoteClientSetFilter(0, filter)
@@ -485,8 +485,8 @@ class DebugViewer(QWidget):
         
         This slot sets the variables filter expressions.
         """
-        self.__setGlobalsFilter()
-        self.__setLocalsFilter()
+        self.setGlobalsFilter()
+        self.setLocalsFilter()
         self.showVariablesTab(False)
         
     def currentWidget(self):
