@@ -560,8 +560,8 @@ class DownloadItem(QWidget, Ui_DownloadItem):
             info = self.tr("{0} of {1} ({2}/sec)\n{3}")\
                 .format(
                     dataString(self.__bytesReceived),
-                    bytesTotal == -1 and self.tr("?")
-                    or dataString(bytesTotal),
+                    bytesTotal == -1 and self.tr("?") or
+                    dataString(bytesTotal),
                     dataString(int(speed)),
                     remaining)
         else:
