@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2011 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -12,7 +12,7 @@ from __future__ import unicode_literals
 import os
 import fnmatch
 
-from PyQt5.QtCore import pyqtSlot, Qt
+from PyQt5.QtCore import pyqtSlot, Qt, QTimer
 from PyQt5.QtGui import QIcon
 from PyQt5.QtWidgets import QDialog, QTreeWidgetItem, QAbstractButton, \
     QDialogButtonBox, QApplication, QHeaderView
@@ -25,7 +25,12 @@ import UI.PixmapCache
 import Preferences
 import Utilities
 
-from . import pep8
+from . import pycodestyle
+
+try:
+    basestring          # __IGNORE_WARNING__
+except Exception:
+    basestring = str    # define for Python3
 
 
 class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
@@ -40,6 +45,14 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
     codeRole = Qt.UserRole + 6
     ignoredRole = Qt.UserRole + 7
     
+    availableFutures = [
+        'division', 'absolute_import', 'with_statement',
+        'print_function', 'unicode_literals', 'generator_stop']
+    
+    noResults = 0
+    noFiles = 1
+    hasResults = 2
+    
     def __init__(self, styleCheckService, parent=None):
         """
         Constructor
@@ -50,6 +63,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         """
         super(CodeStyleCheckerDialog, self).__init__(parent)
         self.setupUi(self)
+        self.setWindowFlags(Qt.Window)
         
         self.excludeMessagesSelectButton.setIcon(
             UI.PixmapCache.getIcon("select.png"))
@@ -63,6 +77,8 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.docTypeComboBox.addItem(self.tr("PEP-257"), "pep257")
         self.docTypeComboBox.addItem(self.tr("Eric"), "eric")
         
+        self.futuresList.addItems(self.availableFutures)
+        
         self.statisticsButton = self.buttonBox.addButton(
             self.tr("Statistics..."), QDialogButtonBox.ActionRole)
         self.statisticsButton.setToolTip(
@@ -85,11 +101,16 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         
         self.styleCheckService = styleCheckService
         self.styleCheckService.styleChecked.connect(self.__processResult)
+        self.styleCheckService.batchFinished.connect(self.__batchFinished)
+        self.styleCheckService.error.connect(self.__processError)
         self.filename = None
         
-        self.noResults = True
+        self.results = CodeStyleCheckerDialog.noResults
         self.cancelled = False
         self.__lastFileItem = None
+        self.__batch = False
+        self.__finished = True
+        self.__errorItem = None
         
         self.__fileOrFileList = ""
         self.__project = None
@@ -97,6 +118,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.__data = {}
         self.__statistics = {}
         self.__onlyFixes = {}
+        self.__noFixCodesList = []
         
         self.on_loadDefaultButton_clicked()
     
@@ -108,12 +130,34 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                                   self.resultList.header().sortIndicatorOrder()
                                   )
     
-    def __createResultItem(self, file, line, pos, message, fixed, autofixing,
-                           ignored):
+    def __createErrorItem(self, filename, message):
+        """
+        Private slot to create a new error item in the result list.
+        
+        @param filename name of the file
+        @type str
+        @param message error message
+        @type str
+        """
+        if self.__errorItem is None:
+            self.__errorItem = QTreeWidgetItem(self.resultList, [
+                self.tr("Errors")])
+            self.__errorItem.setExpanded(True)
+            self.__errorItem.setForeground(0, Qt.red)
+        
+        msg = "{0} ({1})".format(self.__project.getRelativePath(filename),
+                                 message)
+        if not self.resultList.findItems(msg, Qt.MatchExactly):
+            itm = QTreeWidgetItem(self.__errorItem, [msg])
+            itm.setForeground(0, Qt.red)
+            itm.setFirstColumnSpanned(True)
+    
+    def __createResultItem(self, filename, line, pos, message, fixed,
+                           autofixing, ignored):
         """
         Private method to create an entry in the result list.
         
-        @param file file name of the file (string)
+        @param filename file name of the file (string)
         @param line line number of issue (integer or string)
         @param pos character position of issue (integer or string)
         @param message message text (string)
@@ -127,17 +171,18 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         
         if self.__lastFileItem is None:
             # It's a new file
-            self.__lastFileItem = QTreeWidgetItem(self.resultList, [file])
+            self.__lastFileItem = QTreeWidgetItem(self.resultList, [
+                self.__project.getRelativePath(filename)])
             self.__lastFileItem.setFirstColumnSpanned(True)
             self.__lastFileItem.setExpanded(True)
-            self.__lastFileItem.setData(0, self.filenameRole, file)
+            self.__lastFileItem.setData(0, self.filenameRole, filename)
         
         fixable = False
         code, message = message.split(None, 1)
         itm = QTreeWidgetItem(
             self.__lastFileItem,
             ["{0:6}".format(line), code, message])
-        if code.startswith(("W", "-")):
+        if code.startswith(("W", "-", "C", "M")):
             itm.setIcon(1, UI.PixmapCache.getIcon("warning.png"))
         elif code.startswith("N"):
             itm.setIcon(1, UI.PixmapCache.getIcon("namingError.png"))
@@ -147,7 +192,8 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             itm.setIcon(1, UI.PixmapCache.getIcon("syntaxError.png"))
         if fixed:
             itm.setIcon(0, UI.PixmapCache.getIcon("issueFixed.png"))
-        elif code in FixableCodeStyleIssues and not autofixing:
+        elif code in FixableCodeStyleIssues and not autofixing and \
+                code not in self.__noFixCodesList:
             itm.setIcon(0, UI.PixmapCache.getIcon("issueFixable.png"))
             fixable = True
         
@@ -158,7 +204,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         itm.setTextAlignment(1, Qt.AlignVCenter)
         itm.setTextAlignment(2, Qt.AlignVCenter)
         
-        itm.setData(0, self.filenameRole, file)
+        itm.setData(0, self.filenameRole, filename)
         itm.setData(0, self.lineRole, int(line))
         itm.setData(0, self.positionRole, int(pos))
         itm.setData(0, self.messageRole, message)
@@ -253,14 +299,14 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             # initialize the data structure
             self.__data = {
                 "ExcludeFiles": "",
-                "ExcludeMessages": pep8.DEFAULT_IGNORE,
+                "ExcludeMessages": pycodestyle.DEFAULT_IGNORE,
                 "IncludeMessages": "",
                 "RepeatMessages": False,
                 "FixCodes": "",
                 "FixIssues": False,
             }
         if "MaxLineLength" not in self.__data:
-            self.__data["MaxLineLength"] = pep8.MAX_LINE_LENGTH
+            self.__data["MaxLineLength"] = pycodestyle.MAX_LINE_LENGTH
         if "HangClosing" not in self.__data:
             self.__data["HangClosing"] = False
         if "NoFixCodes" not in self.__data:
@@ -269,6 +315,16 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             self.__data["DocstringType"] = "pep257"
         if "ShowIgnored" not in self.__data:
             self.__data["ShowIgnored"] = False
+        if "MaxCodeComplexity" not in self.__data:
+            self.__data["MaxCodeComplexity"] = 10
+        if "ValidEncodings" not in self.__data:
+            self.__data["ValidEncodings"] = "latin-1, utf-8"
+        if "CopyrightMinFileSize" not in self.__data or \
+                "CopyrightAuthor" not in self.__data:
+            self.__data["CopyrightMinFileSize"] = 0
+            self.__data["CopyrightAuthor"] = ""
+        if "FutureChecker" not in self.__data:
+            self.__data["FutureChecker"] = ""
         
         self.excludeFilesEdit.setText(self.__data["ExcludeFiles"])
         self.excludeMessagesEdit.setText(self.__data["ExcludeMessages"])
@@ -282,6 +338,12 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.hangClosingCheckBox.setChecked(self.__data["HangClosing"])
         self.docTypeComboBox.setCurrentIndex(
             self.docTypeComboBox.findData(self.__data["DocstringType"]))
+        self.complexitySpinBox.setValue(self.__data["MaxCodeComplexity"])
+        self.encodingsEdit.setText(self.__data["ValidEncodings"])
+        self.copyrightFileSizeSpinBox.setValue(
+            self.__data["CopyrightMinFileSize"])
+        self.copyrightAuthorEdit.setText(self.__data["CopyrightAuthor"])
+        self.__initFuturesList(self.__data["FutureChecker"])
     
     def start(self, fn, save=False, repeat=None):
         """
@@ -335,6 +397,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                     [f for f in self.files
                      if not fnmatch.fnmatch(f, filter.strip())]
 
+        self.__errorItem = None
         self.__resetStatistics()
         self.__clearErrors(self.files)
         
@@ -350,6 +413,8 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             repeatMessages = self.repeatCheckBox.isChecked()
             fixCodes = self.fixIssuesEdit.text()
             noFixCodes = self.noFixIssuesEdit.text()
+            self.__noFixCodesList = \
+                [c.strip() for c in noFixCodes.split(",") if c.strip()]
             fixIssues = self.fixIssuesCheckBox.isChecked() and repeatMessages
             self.showIgnored = self.ignoredCheckBox.isChecked() and \
                 repeatMessages
@@ -357,16 +422,57 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             hangClosing = self.hangClosingCheckBox.isChecked()
             docType = self.docTypeComboBox.itemData(
                 self.docTypeComboBox.currentIndex())
+            maxCodeComplexity = self.complexitySpinBox.value()
+            miscellaneousArgs = {
+                "CodingChecker": self.encodingsEdit.text(),
+                "CopyrightChecker": {
+                    "MinFilesize": self.copyrightFileSizeSpinBox.value(),
+                    "Author": self.copyrightAuthorEdit.text(),
+                },
+                "FutureChecker": self.__getSelectedFutureImports(),
+            }
             
             self.__options = [excludeMessages, includeMessages, repeatMessages,
                               fixCodes, noFixCodes, fixIssues, maxLineLength,
-                              hangClosing, docType]
+                              hangClosing, docType, maxCodeComplexity,
+                              miscellaneousArgs]
             
             # now go through all the files
             self.progress = 0
             self.files.sort()
-            self.check()
+            if len(self.files) == 1:
+                self.__batch = False
+                self.check()
+            else:
+                self.__batch = True
+                self.checkBatch()
+        else:
+            self.results = CodeStyleCheckerDialog.noFiles
+            self.__finished = False
+            self.__finish()
+    
+    def __modifyOptions(self, source):
+        """
+        Private method to modify the options based on eflag: entries.
         
+        This method looks for comment lines like '# eflag: noqa = M601'
+        at the end of the source in order to extend the list of excluded
+        messages for one file only.
+        
+        @param source source text (list of str or str)
+        @return list of checker options
+        """
+        options = self.__options[:]
+        flags = Utilities.extractFlags(source)
+        if "noqa" in flags and isinstance(flags["noqa"], basestring):
+            excludeMessages = \
+                options[0].strip().rstrip(",")
+            if excludeMessages:
+                excludeMessages += ","
+            excludeMessages += flags["noqa"]
+            options[0] = excludeMessages
+        return options
+    
     def check(self, codestring=''):
         """
         Public method to start a style check for one file.
@@ -394,18 +500,15 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.__lastFileItem = None
         
         if codestring:
-            self.source = codestring
+            source = codestring.splitlines(True)
+            encoding = Utilities.get_coding(source)
         else:
             try:
-                self.source, encoding = Utilities.readEncodedFile(
+                source, encoding = Utilities.readEncodedFile(
                     self.filename)
-                if encoding.endswith(
-                        ('-selected', '-default', '-guessed', '-ignore')):
-                    encoding = encoding.rsplit('-', 1)[0]
-                
-                self.source = self.source.splitlines(True)
+                source = source.splitlines(True)
             except (UnicodeError, IOError) as msg:
-                self.noResults = False
+                self.results = CodeStyleCheckerDialog.hasResults
                 self.__createResultItem(
                     self.filename, 1, 1,
                     self.tr("Error: {0}").format(str(msg))
@@ -414,6 +517,11 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                 # Continue with next file
                 self.check()
                 return
+        if encoding.endswith(
+                ('-selected', '-default', '-guessed', '-ignore')):
+            encoding = encoding.rsplit('-', 1)[0]
+        
+        options = self.__modifyOptions(source)
         
         errors = []
         self.__itms = []
@@ -422,12 +530,92 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             self.__itms.append(itm)
         
         eol = self.__getEol(self.filename)
-        args = self.__options + [
+        args = options + [
             errors, eol, encoding, Preferences.getEditor("CreateBackupFile")
         ]
+        self.__finished = False
         self.styleCheckService.styleCheck(
-            None, self.filename, self.source, args)
-
+            None, self.filename, source, args)
+    
+    def checkBatch(self):
+        """
+        Public method to start a style check batch job.
+        
+        The results are reported to the __processResult slot.
+        """
+        self.__lastFileItem = None
+        
+        self.checkProgressLabel.setPath(self.tr("Preparing files..."))
+        progress = 0
+        
+        argumentsList = []
+        for filename in self.files:
+            progress += 1
+            self.checkProgress.setValue(progress)
+            QApplication.processEvents()
+            
+            try:
+                source, encoding = Utilities.readEncodedFile(
+                    filename)
+                source = source.splitlines(True)
+            except (UnicodeError, IOError) as msg:
+                self.results = CodeStyleCheckerDialog.hasResults
+                self.__createResultItem(
+                    filename, 1, 1,
+                    self.tr("Error: {0}").format(str(msg))
+                        .rstrip(), False, False, False)
+                continue
+            
+            if encoding.endswith(
+                    ('-selected', '-default', '-guessed', '-ignore')):
+                encoding = encoding.rsplit('-', 1)[0]
+            
+            options = self.__modifyOptions(source)
+            
+            errors = []
+            self.__itms = []
+            for error, itm in self.__onlyFixes.pop(filename, []):
+                errors.append(error)
+                self.__itms.append(itm)
+            
+            eol = self.__getEol(filename)
+            args = options + [
+                errors, eol, encoding,
+                Preferences.getEditor("CreateBackupFile")
+            ]
+            argumentsList.append((filename, source, args))
+        
+        # reset the progress bar to the checked files
+        self.checkProgress.setValue(self.progress)
+        self.checkProgressLabel.setPath(self.tr("Transferring data..."))
+        QApplication.processEvents()
+        
+        self.__finished = False
+        self.styleCheckService.styleBatchCheck(argumentsList)
+    
+    def __batchFinished(self):
+        """
+        Private slot handling the completion of a batch job.
+        """
+        self.checkProgressLabel.setPath("")
+        self.checkProgress.setMaximum(1)
+        self.checkProgress.setValue(1)
+        self.__finish()
+    
+    def __processError(self, fn, msg):
+        """
+        Private slot to process an error indication from the service.
+        
+        @param fn filename of the file
+        @type str
+        @param msg error message
+        @type str
+        """
+        self.__createErrorItem(fn, msg)
+        
+        if not self.__batch:
+            self.check()
+    
     def __processResult(self, fn, codeStyleCheckerStats, fixes, results):
         """
         Private slot called after perfoming a style check on one file.
@@ -439,8 +627,12 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             lineno (int), position (int), text (str), ignored (bool),
             fixed (bool), autofixing (bool))
         """
-        # Check if it's the requested file, otherwise ignore signal
-        if fn != self.filename:
+        if self.__finished:
+            return
+        
+        # Check if it's the requested file, otherwise ignore signal if not
+        # in batch mode
+        if not self.__batch and fn != self.filename:
             return
         
         # disable updates of the list for speed
@@ -455,6 +647,8 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                 self.__modifyFixedResultItem(itm, text, fixed)
             self.__updateFixerStatistics(fixes)
         else:
+            self.__lastFileItem = None
+            
             for lineno, position, text, ignored, fixed, autofixing in results:
                 if ignored:
                     ignoredErrors += 1
@@ -462,7 +656,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                         text = self.tr("{0} (ignored)").format(text)
                     else:
                         continue
-                self.noResults = False
+                self.results = CodeStyleCheckerDialog.hasResults
                 self.__createResultItem(
                     fn, lineno, position, text, fixed, autofixing, ignored)
 
@@ -483,36 +677,46 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         self.resultList.setUpdatesEnabled(True)
         
         self.checkProgress.setValue(self.progress)
+        self.checkProgressLabel.setPath(fn)
         QApplication.processEvents()
         
-        self.check()
+        if not self.__batch:
+            self.check()
     
     def __finish(self):
         """
         Private slot called when the code style check finished or the user
         pressed the cancel button.
         """
-        self.cancelled = True
-        self.buttonBox.button(QDialogButtonBox.Close).setEnabled(True)
-        self.buttonBox.button(QDialogButtonBox.Cancel).setEnabled(False)
-        self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
-        self.statisticsButton.setEnabled(True)
-        self.showButton.setEnabled(True)
-        self.startButton.setEnabled(True)
-        
-        if self.noResults:
-            QTreeWidgetItem(self.resultList, [self.tr('No issues found.')])
-            QApplication.processEvents()
-            self.statisticsButton.setEnabled(False)
-            self.showButton.setEnabled(False)
-        else:
+        if not self.__finished:
+            self.__finished = True
+            
+            self.cancelled = True
+            self.buttonBox.button(QDialogButtonBox.Close).setEnabled(True)
+            self.buttonBox.button(QDialogButtonBox.Cancel).setEnabled(False)
+            self.buttonBox.button(QDialogButtonBox.Close).setDefault(True)
             self.statisticsButton.setEnabled(True)
             self.showButton.setEnabled(True)
-        self.resultList.header().resizeSections(QHeaderView.ResizeToContents)
-        self.resultList.header().setStretchLastSection(True)
-        
-        self.checkProgress.setVisible(False)
-        self.checkProgressLabel.setVisible(False)
+            self.startButton.setEnabled(True)
+            
+            if self.results != CodeStyleCheckerDialog.hasResults:
+                if self.results == CodeStyleCheckerDialog.noResults:
+                    QTreeWidgetItem(
+                        self.resultList, [self.tr('No issues found.')])
+                else:
+                    QTreeWidgetItem(
+                        self.resultList,
+                        [self.tr('No files found (check your ignore list).')])
+                QApplication.processEvents()
+                self.showButton.setEnabled(False)
+            else:
+                self.showButton.setEnabled(True)
+            self.resultList.header().resizeSections(
+                QHeaderView.ResizeToContents)
+            self.resultList.header().setStretchLastSection(True)
+            
+            self.checkProgress.setVisible(False)
+            self.checkProgressLabel.setVisible(False)
     
     def __getEol(self, fn):
         """
@@ -546,6 +750,11 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                 "HangClosing": self.hangClosingCheckBox.isChecked(),
                 "DocstringType": self.docTypeComboBox.itemData(
                     self.docTypeComboBox.currentIndex()),
+                "MaxCodeComplexity": self.complexitySpinBox.value(),
+                "ValidEncodings": self.encodingsEdit.text(),
+                "CopyrightMinFileSize": self.copyrightFileSizeSpinBox.value(),
+                "CopyrightAuthor": self.copyrightAuthorEdit.text(),
+                "FutureChecker": self.__getSelectedFutureImports(),
             }
             if data != self.__data:
                 self.__data = data
@@ -553,7 +762,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
                                        self.__data)
         
         self.resultList.clear()
-        self.noResults = True
+        self.results = CodeStyleCheckerDialog.noResults
         self.cancelled = False
         self.start(self.__fileOrFileList)
     
@@ -610,7 +819,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         @param item reference to the activated item (QTreeWidgetItem)
         @param column column the item was activated in (integer)
         """
-        if self.noResults:
+        if self.results != CodeStyleCheckerDialog.hasResults:
             return
         
         if item.parent():
@@ -629,6 +838,8 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             else:
                 editor.toggleWarning(
                     lineno, 0, True, message, warningType=editor.WarningStyle)
+            
+            editor.updateVerticalScrollBar()
     
     @pyqtSlot()
     def on_resultList_itemSelectionChanged(self):
@@ -675,6 +886,9 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
             if file not in errorFiles:
                 editor = vm.getOpenEditor(file)
                 editor.clearStyleWarnings()
+        
+        editor = vm.activeWindow()
+        editor.updateVerticalScrollBar()
     
     @pyqtSlot()
     def on_statisticsButton_clicked(self):
@@ -691,27 +905,37 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         Private slot to load the default configuration values.
         """
         self.excludeFilesEdit.setText(Preferences.Prefs.settings.value(
-            "PEP8/ExcludeFilePatterns"))
+            "PEP8/ExcludeFilePatterns", ""))
         self.excludeMessagesEdit.setText(Preferences.Prefs.settings.value(
-            "PEP8/ExcludeMessages", pep8.DEFAULT_IGNORE))
+            "PEP8/ExcludeMessages", pycodestyle.DEFAULT_IGNORE))
         self.includeMessagesEdit.setText(Preferences.Prefs.settings.value(
-            "PEP8/IncludeMessages"))
+            "PEP8/IncludeMessages", ""))
         self.repeatCheckBox.setChecked(Preferences.toBool(
-            Preferences.Prefs.settings.value("PEP8/RepeatMessages")))
+            Preferences.Prefs.settings.value("PEP8/RepeatMessages", False)))
         self.fixIssuesEdit.setText(Preferences.Prefs.settings.value(
-            "PEP8/FixCodes"))
+            "PEP8/FixCodes", ""))
         self.noFixIssuesEdit.setText(Preferences.Prefs.settings.value(
             "PEP8/NoFixCodes", "E501"))
         self.fixIssuesCheckBox.setChecked(Preferences.toBool(
-            Preferences.Prefs.settings.value("PEP8/FixIssues")))
+            Preferences.Prefs.settings.value("PEP8/FixIssues", False)))
         self.ignoredCheckBox.setChecked(Preferences.toBool(
-            Preferences.Prefs.settings.value("PEP8/ShowIgnored")))
+            Preferences.Prefs.settings.value("PEP8/ShowIgnored", False)))
         self.lineLengthSpinBox.setValue(int(Preferences.Prefs.settings.value(
-            "PEP8/MaxLineLength", pep8.MAX_LINE_LENGTH)))
+            "PEP8/MaxLineLength", pycodestyle.MAX_LINE_LENGTH)))
         self.hangClosingCheckBox.setChecked(Preferences.toBool(
-            Preferences.Prefs.settings.value("PEP8/HangClosing")))
+            Preferences.Prefs.settings.value("PEP8/HangClosing", False)))
         self.docTypeComboBox.setCurrentIndex(self.docTypeComboBox.findData(
             Preferences.Prefs.settings.value("PEP8/DocstringType", "pep257")))
+        self.complexitySpinBox.setValue(int(Preferences.Prefs.settings.value(
+            "PEP8/MaxCodeComplexity", 10)))
+        self.encodingsEdit.setText(Preferences.Prefs.settings.value(
+            "PEP8/ValidEncodings", "latin-1, utf-8"))
+        self.copyrightFileSizeSpinBox.setValue(int(
+            Preferences.Prefs.settings.value("PEP8/CopyrightMinFileSize", 0)))
+        self.copyrightAuthorEdit.setText(
+            Preferences.Prefs.settings.value("PEP8/CopyrightAuthor", ""))
+        self.__initFuturesList(
+            Preferences.Prefs.settings.value("PEP8/FutureChecker", ""))
     
     @pyqtSlot()
     def on_storeDefaultButton_clicked(self):
@@ -742,6 +966,16 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         Preferences.Prefs.settings.setValue(
             "PEP8/DocstringType", self.docTypeComboBox.itemData(
                 self.docTypeComboBox.currentIndex()))
+        Preferences.Prefs.settings.setValue(
+            "PEP8/MaxCodeComplexity", self.complexitySpinBox.value())
+        Preferences.Prefs.settings.setValue(
+            "PEP8/ValidEncodings", self.encodingsEdit.text())
+        Preferences.Prefs.settings.setValue(
+            "PEP8/CopyrightMinFileSize", self.copyrightFileSizeSpinBox.value())
+        Preferences.Prefs.settings.setValue(
+            "PEP8/CopyrightAuthor", self.copyrightAuthorEdit.text())
+        Preferences.Prefs.settings.setValue(
+            "PEP8/FutureChecker", self.__getSelectedFutureImports())
     
     @pyqtSlot()
     def on_resetDefaultButton_clicked(self):
@@ -750,7 +984,7 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         """
         Preferences.Prefs.settings.setValue("PEP8/ExcludeFilePatterns", "")
         Preferences.Prefs.settings.setValue(
-            "PEP8/ExcludeMessages", pep8.DEFAULT_IGNORE)
+            "PEP8/ExcludeMessages", pycodestyle.DEFAULT_IGNORE)
         Preferences.Prefs.settings.setValue("PEP8/IncludeMessages", "")
         Preferences.Prefs.settings.setValue("PEP8/RepeatMessages", False)
         Preferences.Prefs.settings.setValue("PEP8/FixCodes", "")
@@ -758,9 +992,15 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         Preferences.Prefs.settings.setValue("PEP8/FixIssues", False)
         Preferences.Prefs.settings.setValue("PEP8/ShowIgnored", False)
         Preferences.Prefs.settings.setValue(
-            "PEP8/MaxLineLength", pep8.MAX_LINE_LENGTH)
+            "PEP8/MaxLineLength", pycodestyle.MAX_LINE_LENGTH)
         Preferences.Prefs.settings.setValue("PEP8/HangClosing", False)
         Preferences.Prefs.settings.setValue("PEP8/DocstringType", "pep257")
+        Preferences.Prefs.settings.setValue("PEP8/MaxCodeComplexity", 10)
+        Preferences.Prefs.settings.setValue(
+            "PEP8/ValidEncodings", "latin-1, utf-8")
+        Preferences.Prefs.settings.setValue("PEP8/CopyrightMinFileSize", 0)
+        Preferences.Prefs.settings.setValue("PEP8/CopyrightAuthor", "")
+        Preferences.Prefs.settings.setValue("PEP8/FutureChecker", "")
     
     @pyqtSlot(QAbstractButton)
     def on_buttonBox_clicked(self, button):
@@ -772,7 +1012,11 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         if button == self.buttonBox.button(QDialogButtonBox.Close):
             self.close()
         elif button == self.buttonBox.button(QDialogButtonBox.Cancel):
-            self.__finish()
+            if self.__batch:
+                self.styleCheckService.cancelStyleBatchCheck()
+                QTimer.singleShot(1000, self.__finish)
+            else:
+                self.__finish()
         elif button == self.showButton:
             self.on_showButton_clicked()
         elif button == self.statisticsButton:
@@ -856,3 +1100,30 @@ class CodeStyleCheckerDialog(QDialog, Ui_CodeStyleCheckerDialog):
         """
         return (itm.data(0, self.fixableRole) and
                 not itm.data(0, self.ignoredRole))
+    
+    def __initFuturesList(self, selectedFutures):
+        """
+        Private method to set the selected status of the future imports.
+        
+        @param selectedFutures comma separated list of expected future imports
+        @type str
+        """
+        if selectedFutures:
+            expectedImports = [
+                i.strip() for i in selectedFutures.split(",")
+                if bool(i.strip())]
+        else:
+            expectedImports = []
+        for row in range(self.futuresList.count()):
+            itm = self.futuresList.item(row)
+            itm.setSelected(itm.text() in expectedImports)
+    
+    def __getSelectedFutureImports(self):
+        """
+        Private method to get the expected future imports.
+        
+        @return expected future imports as a comma separated string
+        @rtype str
+        """
+        selectedFutures = [i.text() for i in self.futuresList.selectedItems()]
+        return ", ".join(selectedFutures)
