@@ -141,21 +141,10 @@ class HgLogDialog(QWidget, Ui_HgLogDialog):
                 args.append("--subrepos")
         if self.mode == "log":
             args.append('--copies')
-        if self.vcs.version >= (3, 0):
-            args.append('--template')
-            args.append(os.path.join(os.path.dirname(__file__),
-                                     "templates",
-                                     "logDialogBookmarkPhase.tmpl"))
-        else:
-            args.append('--style')
-            if self.vcs.version >= (2, 1):
-                args.append(os.path.join(os.path.dirname(__file__),
-                                         "styles",
-                                         "logDialogBookmarkPhase.style"))
-            else:
-                args.append(os.path.join(os.path.dirname(__file__),
-                                         "styles",
-                                         "logDialogBookmark.style"))
+        args.append('--template')
+        args.append(os.path.join(os.path.dirname(__file__),
+                                 "templates",
+                                 "logDialogBookmarkPhase.tmpl"))
         if self.mode == "incoming":
             if self.bundle:
                 args.append(self.bundle)
