@@ -82,8 +82,8 @@ class HgProjectBrowserHelper(VcsProjectBrowserHelper):
                     # adjust individual extension menu entries
                     self.__extensions[extensionName].showExtensionMenu(
                         key, controlled)
-                if (not self.__extensionMenus[extensionName][key].isEnabled()
-                    and self.__extensionMenus[extensionName][key]
+                if (not self.__extensionMenus[extensionName][key]
+                    .isEnabled() and self.__extensionMenus[extensionName][key]
                         .isTearOffMenuVisible()):
                     self.__extensionMenus[extensionName][key].hideTearOffMenu()
     
