@@ -9,18 +9,28 @@ Module implementing the Help Viewers configuration page.
 
 from __future__ import unicode_literals
 
-from PyQt5.QtCore import pyqtSlot
+from PyQt5.QtCore import qVersion
 from PyQt5.QtWidgets import QButtonGroup
+try:
+    from PyQt5 import QtWebKit      # __IGNORE_WARNING__
+    WEBKIT_AVAILABLE = True
+except ImportError:
+    WEBKIT_AVAILABLE = False
+if qVersion() < "5.6.0":
+    WEBENGINE_AVAILABLE = False
+else:
+    try:
+        from PyQt5 import QtWebEngineWidgets    # __IGNORE_WARNING__
+        WEBENGINE_AVAILABLE = True
+    except ImportError:
+        WEBENGINE_AVAILABLE = False
 
-from E5Gui.E5Completers import E5FileCompleter
-from E5Gui import E5FileDialog
+from E5Gui.E5PathPicker import E5PathPickerModes
 
 from .ConfigurationPageBase import ConfigurationPageBase
 from .Ui_HelpViewersPage import Ui_HelpViewersPage
 
 import Preferences
-import Utilities
-import UI.PixmapCache
 
 
 class HelpViewersPage(ConfigurationPageBase, Ui_HelpViewersPage):
@@ -35,8 +45,7 @@ class HelpViewersPage(ConfigurationPageBase, Ui_HelpViewersPage):
         self.setupUi(self)
         self.setObjectName("HelpViewersPage")
         
-        self.customViewerSelectionButton.setIcon(
-            UI.PixmapCache.getIcon("open.png"))
+        self.customViewerPicker.setMode(E5PathPickerModes.OpenFileMode)
         
         self.helpViewerGroup = QButtonGroup()
         self.helpViewerGroup.addButton(self.helpBrowserButton)
@@ -44,17 +53,15 @@ class HelpViewersPage(ConfigurationPageBase, Ui_HelpViewersPage):
         self.helpViewerGroup.addButton(self.webBrowserButton)
         self.helpViewerGroup.addButton(self.customViewerButton)
         
-        self.customViewerCompleter = E5FileCompleter(self.customViewerEdit)
-        
         # set initial values
-        hvId = Preferences.getHelp("HelpViewerType")
-        # check availability of QtWebKit
-        try:
-            from PyQt5 import QtWebKit      # __IGNORE_WARNING__
-        except ImportError:
-            # not available, reset help viewer to default
+        if WEBENGINE_AVAILABLE:
+            hvId = Preferences.getWebBrowser("HelpViewerType")
+        else:
+            hvId = Preferences.getHelp("HelpViewerType")
+        if not WEBENGINE_AVAILABLE and not WEBKIT_AVAILABLE:
             if hvId == 1:
-                hvId = Preferences.Prefs.helpDefaults["HelpViewerType"]
+                hvId = Preferences.Prefs.webBrowserDefaultsDefaults[
+                    "HelpViewerType"]
             self.helpBrowserButton.setEnabled(False)
         
         if hvId == 1:
@@ -65,7 +72,7 @@ class HelpViewersPage(ConfigurationPageBase, Ui_HelpViewersPage):
             self.webBrowserButton.setChecked(True)
         else:
             self.customViewerButton.setChecked(True)
-        self.customViewerEdit.setText(
+        self.customViewerPicker.setText(
             Preferences.getHelp("CustomViewer"))
         
     def save(self):
@@ -81,65 +88,10 @@ class HelpViewersPage(ConfigurationPageBase, Ui_HelpViewersPage):
         elif self.customViewerButton.isChecked():
             hvId = 4
         Preferences.setHelp("HelpViewerType", hvId)
+        Preferences.setWebBrowser("HelpViewerType", hvId)
         Preferences.setHelp(
             "CustomViewer",
-            self.customViewerEdit.text())
-        
-    @pyqtSlot()
-    def on_customViewerSelectionButton_clicked(self):
-        """
-        Private slot to handle the custom viewer selection.
-        """
-        file = E5FileDialog.getOpenFileName(
-            self,
-            self.tr("Select Custom Viewer"),
-            self.customViewerEdit.text(),
-            "")
-        
-        if file:
-            self.customViewerEdit.setText(Utilities.toNativeSeparators(file))
-        
-    @pyqtSlot()
-    def on_webbrowserButton_clicked(self):
-        """
-        Private slot to handle the Web browser selection.
-        """
-        file = E5FileDialog.getOpenFileName(
-            self,
-            self.tr("Select Web-Browser"),
-            self.webbrowserEdit.text(),
-            "")
-        
-        if file:
-            self.webbrowserEdit.setText(Utilities.toNativeSeparators(file))
-        
-    @pyqtSlot()
-    def on_pdfviewerButton_clicked(self):
-        """
-        Private slot to handle the PDF viewer selection.
-        """
-        file = E5FileDialog.getOpenFileName(
-            self,
-            self.tr("Select PDF-Viewer"),
-            self.pdfviewerEdit.text(),
-            "")
-        
-        if file:
-            self.pdfviewerEdit.setText(Utilities.toNativeSeparators(file))
-        
-    @pyqtSlot()
-    def on_chmviewerButton_clicked(self):
-        """
-        Private slot to handle the CHM viewer selection.
-        """
-        file = E5FileDialog.getOpenFileName(
-            self,
-            self.tr("Select CHM-Viewer"),
-            self.chmviewerEdit.text(),
-            "")
-        
-        if file:
-            self.chmviewerEdit.setText(Utilities.toNativeSeparators(file))
+            self.customViewerPicker.text())
     
 
 def create(dlg):
