@@ -678,6 +678,28 @@ def extractLineFlags(line, startComment="#", endComment=""):
     return flags
 
 
+def filterAnsiSequences(txt):
+    """
+    Function to filter out ANSI escape sequences (color only).
+    
+    @param txt text to be filtered
+    @type str
+    @return text without ANSI escape sequences
+    @rtype str
+    """
+    ntxt = txt[:]
+    while True:
+        start = ntxt.find("\33[")    # find escape character
+        if start == -1:
+            break
+        end = ntxt.find("m", start)
+        if end == -1:
+            break
+        ntxt = ntxt[:start] + ntxt[end + 1:]
+    
+    return ntxt
+
+
 def toNativeSeparators(path):
     """
     Function returning a path, that is using native separator characters.
@@ -1682,6 +1704,14 @@ def generateVersionInfo(linesep='\n'):
         qVersion(), linesep, PYQT_VERSION_STR, linesep)
     info += "  sip {0}{1}  QScintilla {2}{3}".format(
         sip_version_str, linesep, QSCINTILLA_VERSION_STR, linesep)
+    if qVersion() >= "5.6.0":
+        try:
+            from PyQt5 import QtWebEngineWidgets    # __IGNORE_WARNING__
+            from WebBrowser.Tools import WebBrowserTools
+            chromeVersion = WebBrowserTools.getWebEngineVersions()[0]
+            info += "  WebEngine {0}{1}".format(chromeVersion, linesep)
+        except ImportError:
+            pass
     try:
         from PyQt5.QtWebKit import qWebKitVersion
         info += "  WebKit {0}{1}".format(qWebKitVersion(), linesep)
